@@ -13,13 +13,15 @@ use 5.006001;
 use strict;
 use warnings;
 
-use Pegex 0.13 ();
+my $requires = "
+use Pegex 0.19 ();
+";
 
 package TestML;
 
 use TestML::Runtime;
 
-our $VERSION = '0.24';
+our $VERSION = '0.25';
 
 use constant XXX_skip => 1;
 our $DumpModule = 'YAML::XS';
