@@ -26,16 +26,6 @@ sub compile {
     return $self;
 }
 
-sub compile_raw {
-    my $self = shift;
-    $self = $self->new unless ref $self;
-
-    $self->parse(shift);
-    $self->combinate;
-
-    return $self;
-}
-
 sub parse {
     if ($Pegex::Bootstrap) {
         require Pegex::Bootstrap;
@@ -121,8 +111,9 @@ sub combinate_re {
     my $regexp = shift;
     my $atoms = Pegex::Grammar::Atoms->atoms;
     $regexp->{'.rgx'} =~ s!~!<ws>!g;
+    my $re = $regexp->{'.rgx'};
+    $re =~ s!~!<ws>!g;
     while (1) {
-        my $re = $regexp->{'.rgx'};
         $re =~ s[<(\w+)>][
             $self->tree->{$1} and
             $self->tree->{$1}{'.rgx'}
