@@ -33,17 +33,28 @@ my %prefixes = (
 
 sub got_grammar {
     my ($self, $rules) = @_;
+    my ($meta_section, $rule_section) = @$rules;
     my $grammar = {
         '+top' => $self->top,
         %{$self->extra_rules},
+        %$meta_section,
     };
-    for (@$rules) {
+    for (@$rule_section) {
         my ($key, $value) = %$_;
         $grammar->{$key} = $value;
     }
     return $grammar;
 }
 
+sub got_meta_section {
+    my ($self, $directives) = @_;
+    my $meta = {};
+    for my $next (@$directives) {
+        $meta->{"+$next->[0]"} = $next->[1];
+    }
+    return $meta;
+}
+
 sub got_rule_definition {
     my ($self, $match) = @_;
     my $name = $match->[0];
@@ -104,7 +115,9 @@ sub got_rule_part {
     my ($rule, $sep_op, $sep_rule) = @$part;
     if ($sep_rule) {
         $sep_rule->{'+eok'} = 1
-            if $sep_op eq '%%';
+            if $sep_op =~ /^%%%?$/;
+        $sep_rule->{'+bok'} = 1
+            if $sep_op eq '%%%';
         $rule->{'.sep'} = $sep_rule;
     }
     return $rule;
