@@ -42,28 +42,55 @@ sub tree_ {
     'comment' => {
       '.rgx' => qr/(?-xism:\G(?:[\ \t]*\r?\n|\#.*\r?\n))/
     },
+    'ending' => {
+      '.rgx' => qr/(?-xism:\G\s*?(?:\n\s*|;\s*|\z))/
+    },
     'error_message' => {
       '.rgx' => qr/(?-xism:\G`([^`\r\n]*)`)/
     },
     'grammar' => {
       '.all' => [
         {
-          '+min' => 0,
-          '-skip' => 1,
-          '.ref' => 'comment'
+          '.ref' => 'meta_section'
         },
         {
-          '+min' => 1,
-          '.ref' => 'rule_definition',
-          '.sep' => {
-            '+eok' => 1,
-            '+min' => 0,
-            '-skip' => 1,
-            '.ref' => 'comment'
-          }
+          '.ref' => 'rule_section'
         }
       ]
     },
+    'meta_definition' => {
+      '.all' => [
+        {
+          '.rgx' => qr/(?-xism:\G%)/
+        },
+        {
+          '.ref' => 'meta_name'
+        },
+        {
+          '.rgx' => qr/(?-xism:\G[\ \t]+)/
+        },
+        {
+          '.ref' => 'meta_value'
+        }
+      ]
+    },
+    'meta_name' => {
+      '.rgx' => qr/(?-xism:\G(grammar|extends|include|version))/
+    },
+    'meta_section' => {
+      '+min' => 0,
+      '.ref' => 'meta_definition',
+      '.sep' => {
+        '+bok' => 1,
+        '+eok' => 1,
+        '+min' => 0,
+        '-skip' => 1,
+        '.ref' => 'comment'
+      }
+    },
+    'meta_value' => {
+      '.rgx' => qr/(?-xism:\G[\ \t]*([^;\n]*?)[\ \t]*\s*?(?:\n\s*|;\s*|\z))/
+    },
     'regular_expression' => {
       '.rgx' => qr/(?-xism:\G\/([^\/\r\n]*)\/)/
     },
@@ -82,13 +109,10 @@ sub tree_ {
           '.ref' => 'rule_group'
         },
         {
-          '.ref' => 'rule_ending'
+          '.ref' => 'ending'
         }
       ]
     },
-    'rule_ending' => {
-      '.rgx' => qr/(?-xism:\G\s*?(?:\n\s*|;\s*|\z))/
-    },
     'rule_group' => {
       '.any' => [
         {
@@ -123,11 +147,22 @@ sub tree_ {
       '+min' => '1',
       '.ref' => 'rule_item',
       '.sep' => {
-        '.rgx' => qr/(?-xism:\G\s*\s(%%?)\s\s*)/
+        '.rgx' => qr/(?-xism:\G\s*\s(%{1,3})\s\s*)/
       }
     },
     'rule_reference' => {
       '.rgx' => qr/(?-xism:\G([!=\+\-\.]?)<([a-zA-Z]\w*)>((?:[\*\+\?]|[0-9]+(?:\-[0-9]+|\+)?)?))/
+    },
+    'rule_section' => {
+      '+min' => 1,
+      '.ref' => 'rule_definition',
+      '.sep' => {
+        '+bok' => 1,
+        '+eok' => 1,
+        '+min' => 0,
+        '-skip' => 1,
+        '.ref' => 'comment'
+      }
     }
   }
 }
